from typing import TypedDict

from .utils.Classes.String import String
from .utils.assert_string import assert_string
from .utils.merge import merge

emoji_patterns = (
    "^["
    r"\U0001F1E0-\U0001F1FF"  # flags (iOS)
    r"\U0001F300-\U0001F5FF"  # symbols & pictographs
    r"\U0001F600-\U0001F64F"  # emoticons
    r"\U0001F680-\U0001F6FF"  # transport & map symbols
    r"\U0001F700-\U0001F77F"  # alchemical symbols
    r"\U0001F780-\U0001F7FF"  # Geometric Shapes Extended
    r"\U0001F800-\U0001F8FF"  # Supplemental Arrows-C
    r"\U0001F900-\U0001F9FF"  # Supplemental Symbols and Pictographs
    r"\U0001FA00-\U0001FA6F"  # Chess Symbols
    r"\U0001FA70-\U0001FAFF"  # Symbols and Pictographs Extended-A
    r"\U00002702-\U000027B0"  # Dingbats
    "]$"
)


class IsEmojiOptions(TypedDict):
    omit_rule: str


default_emoji_options: IsEmojiOptions = {
    "omit_rule": None,
}


def is_emoji(input: str, options: IsEmojiOptions = {}) -> bool:
    input = assert_string(input)

    options = merge(options, default_emoji_options)

    def is_flag_emoji(c):
        return (
            r"\U0001F1E6\U0001F1E8" <= c <= r"\U0001F1FF\U0001F1FC"
            or (
                c in [
                    r"\U0001F3F4\U000e0067\U000e0062\U000e0065\U000e006e\U000e0067\U000e007f",
                    r"\U0001F3F4\U000e0067\U000e0062\U000e0073\U000e0063\U000e0074\U000e007f",
                    r"\U0001F3F4\U000e0067\U000e0062\U000e0077\U000e006c\U000e0073\U000e007f"
                ]
            )
        )

    if options["omit_rule"] is not None:
        input = input.sub(options["omit_rule"], '')

    if not input.length:
        return False

    for char in input:
        char = String(char)
        if not (char.match(emoji_patterns) or is_flag_emoji(char)):
            return False

    return True



from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string


def is_md5(input: str) -> bool:
    input = assert_string(input)

    md5_pattern = RegEx("^[a-f0-9]{32}$")

    return md5_pattern.match(input)



from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string


def is_semantic_version(input: str) -> bool:
    input = assert_string(input)

    semantic_version_pattern = RegEx(
        r"^(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)" +
        r"(?:-((?:0|[1-9]\d*|\d*[a-z-][0-9a-z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-z-][0-9a-z-]*))*))" +
        r"?(?:\+([0-9a-z-]+(?:\.[0-9a-z-]+)*))?$", "i"
    )

    return semantic_version_pattern.match(input)



from typing import TypedDict

from .utils.assert_string import assert_string


class IsHtmlElementOptions(TypedDict):
    contains: bool


default_email_options: IsHtmlElementOptions = {
    "contains": False
}


def is_html_element(input: str, options: IsHtmlElementOptions = default_email_options) -> bool:
    input = assert_string(input).trim()

    html_contains_pattern = r"<\s*?[^/<>]*(!DOCTYPE|!--|br|basefont|hr|source|frame|param|area|meta|col|link|option|base|img|wbr)[^<>]*?>|<\s*?[^<>]*input\s*?[^<>]*/>|<\s*?[^<>]*(a|abbr|acronym|address|applet|article|aside|audio|body|b|bdi|bdo|big|blockquote|button|canvas|caption|center|cite|code|colgroup|command|datalist|dd|del|details|dfn|dialog|dir|div|dl|dt|em|embed|fieldset|figcaption|figure|font|footer|form|frameset|head|header|hgroup|h1|h2|h3|h4|h5|h6|html|i|iframe|ins|kbd|keygen|label|legend|li|map|mark|menu|meter|noframes|nav|noscript|object|ol|optgroup|output|p|pre|progress|q|rp|rt|ruby|s|samp|script|section|select|small|span|strike|strong|style|sub|summary|sup|table|tbody|td|textarea|tfoot|th|thead|time|title|tr|track|tt|u|ul|var|video)(((\s+)[a-z\-A-Z]*=\".*\")|((\s+)[a-zA-Z])*)?(\s+)?>[\s\S]*?(<(\s+)?/(\s+)?\2(\s+)?>)"
    html_strict = "^{}$".format(html_contains_pattern)

    if options["contains"]:
        index = input.search(html_contains_pattern)
        return True if index is not None else False

    return input.match(html_strict)



import unittest
import inspect

from pyvalidator.utils.to_string import to_string, obj_str
from . import print_test_ok


class TestIsUuid(unittest.TestCase):

    def test_input_str(self):
        self.assertEqual(to_string("x"), "x")
        print_test_ok()

    def test_input_int(self):
        self.assertEqual(to_string(1), "1")
        print_test_ok()

    def test_input_float(self):
        self.assertEqual(to_string(float(1.1)), "1.1")
        print_test_ok()

    def test_input_dict(self):
        self.assertEqual(to_string({"test": 1}), obj_str)
        print_test_ok()

    def test_input_list(self):
        self.assertEqual(to_string([1]), obj_str)
        print_test_ok()

    def test_input_module(self):
        self.assertEqual(to_string(inspect), obj_str)
        print_test_ok()

    def test_input_undefined(self):
        self.assertEqual(to_string(type), '')
        print_test_ok()



import re
from typing import AnyStr, Literal, Pattern, Union

from .List import List

FlagType = Union[Literal["g"], Literal["i"], Literal["m"], Literal["u"], None]


class RegEx(object):
    pattern = None
    raw_pattern = None

    def __init__(self, pattern: str, flag: FlagType = None) -> None:
        raw_pattern = pattern
        if not pattern:
            raise ValueError("Pattern not provided")
        self.compile_pattern(pattern, flag)

    def compile_pattern(self, pattern: str, flag: FlagType) -> None:
        if flag == 'i':
            self.pattern = re.compile(pattern, re.IGNORECASE)
        elif flag == 'g':
            self.pattern = re.compile(pattern, re.DOTALL)
        elif flag == 'm':
            self.pattern = re.compile(pattern, re.MULTILINE)
        elif flag == 'u':
            self.pattern = re.compile(pattern, re.UNICODE)
        else:
            self.pattern = re.compile(pattern)

    def match(self, input: str) -> bool:
        if not self.pattern:
            return False
        return bool(self.pattern.match(input))

    def split(self, input: str) -> List:
        return List(re.split(self.pattern, input))

    @staticmethod
    def sub(pattern: str, replacement: str, input: str) -> str:
        return re.sub(pattern, replacement, input)

    @staticmethod
    def escape(input: str) -> str:
        return re.escape(input)

    def findall(self, target: str) -> Union[List, None]:
        matches = re.findall(self.pattern, target)
        if not len(matches):
            return None
        return re.findall(self.pattern, target)

    @staticmethod
    def compile(pattern) -> Pattern[AnyStr]:
        return re.compile(pattern)

    def search(self, target: str) -> Union[int, None]:
        return re.search(self.pattern, target).start()



import unittest

from pyvalidator.is_boolean import is_boolean
from . import print_test_ok


class TestIsBoolean(unittest.TestCase):

    def test_valid_booleans(self):
        for i in [
            'True',
            'False',
            'true',
            'false',
            '0',
            '1',
        ]:
            self.assertTrue(is_boolean(i))
        print_test_ok()

    def test_invalid_booleans(self):
        for i in [
            '1.0',
            '0.0',
            'true ',
            'yes',
        ]:
            self.assertFalse(is_boolean(i))
        print_test_ok()

    def test_valid_loosly_booleans(self):
        for i in [
            'true',
            'True',
            'TRUE',
            'false',
            'False',
            'FALSE',
            '0',
            '1',
            'yes',
            'Yes',
            'YES',
            'no',
            'No',
            'NO',
        ]:
            self.assertTrue(is_boolean(i, {"loose": True}))
        print_test_ok()

    def test_invalid_loosly_booleans(self):
        for i in [
            '1.0',
            '0.0',
            'true ',
            ' false',
        ]:
            self.assertFalse(is_boolean(i, {"loose": True}))
        print_test_ok()



from typing import TypedDict

from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string
from .utils.merge import merge


class IsEmailOptions(TypedDict):
    allow_hyphens: bool


default_email_options: IsEmailOptions = {
    "allow_hyphens": False,
}


def is_imei(input: str, options={}) -> bool:
    input = assert_string(input)
    options = merge(options, default_email_options)

    imei_regex_without_hyphens = r"^[0-9]{15}$"
    imei_regex_with_hyphens = r"^\d{2}-\d{6}-\d{6}-\d{1}$"

    imei_regex = imei_regex_with_hyphens if options["allow_hyphens"] else imei_regex_without_hyphens

    if not input.match(imei_regex):
        return False

    input = input.sub(RegEx('-', 'g'), '')

    sum = 0
    mul = 2
    l = 14

    for i in range(14):
        digit = input.substring(l - i - 1, l - i)
        tp = int(digit) * mul
        if tp >= 10:
            sum += (tp % 10) + 1
        else:
            sum += tp
        if mul == 1:
            mul += 1
        else:
            mul -= 1

    chk = ((10 - (sum % 10)) % 10)

    if chk != int(input.substring(14, 15)):
        return False
    return True



import unittest

from pyvalidator.is_base32 import is_base32
from . import print_test_ok


class TestIsBase32(unittest.TestCase):

    def test_valid_base32(self):
        for i in [
            'ZG======',
            'JBSQ====',
            'JBSWY===',
            'K5SWYY3PNVSSA5DPEBXG6ZA=',
            'K5SWYY3PNVSSA5DPEBXG6===',
        ]:
            self.assertTrue(is_base32(i))
        print_test_ok()

    def test_invalid_base32(self):
        for i in [
            '12345',
            '',
            'ZG=====',
            'Zm=8JBSWY3DP',
            '=m9vYg==',
            'Zm9vYm/y====',
        ]:
            self.assertFalse(is_base32(i))
        print_test_ok()



from .utils.to_date import default_date_string, to_date


def is_before(input, date=default_date_string()):
    comparison = to_date(date)
    original = to_date(input)
    return bool(original and comparison and original < comparison)



import unittest

from pyvalidator.is_strong_password import is_strong_password
from . import print_test_ok


class TestIsStrongPassword(unittest.TestCase):
    options = {
        "min_length": 8,
        "min_lowercase": 1,
        "min_uppercase": 1,
        "min_numbers": 1,
        "min_symbols": 1,
    }

    def test_valid_stong_password(self):
        for i in [
            '%2%k{7BsL"M%Kd6e',
            'EXAMPLE of very long_password123!',
            'mxH_+2vs&54_+H3P',
            '+&DxJ=X7-4L8jRCD',
            'etV*p%Nr6w&H%FeF',
        ]:
            self.assertTrue(is_strong_password(i, self.options))
        print_test_ok()

    def test_invalid_stong_password(self):
        for i in [
            '',
            'password',
            'hunter2',
            'hello world',
            'passw0rd',
            'password!',
            'PASSWORD!',
        ]:
            self.assertFalse(is_strong_password(i, self.options))
        print_test_ok()



import unittest

from pyvalidator.is_aws_arn import is_aws_arn
from . import print_test_ok


class TestIsAWSArn(unittest.TestCase):

    def test_valid_aws_arn(self):
        for i in [
			'arn:aws:iam::123456789012:user/someName1',
			'arn:aws:iam::123456789012:user/someName2',
			'arn:aws:iam::123456789012:user/user@mail.com',
			'arn:aws:iam::123456789012:user/*',
			'arn:aws:iam::123456789012:user/Development/product_1234/*',
			'arn:aws:iam::123456789012:group/groupName1',
			'arn:aws:iam::123456789012:group/groupName2',
			'arn:aws:iam::123456789012:group/*',
			'arn:aws:iam::123456789012:group/Development/product_1234/*',
			'arn:aws:iam::123456789012:group/division_abc/subdivision_xyz/product_A/Developers',
			'arn:aws:iam::123456789012:group/Developers',
			'arn:aws:iam::123456789012:role/TestRole',
			'arn:aws:iam::123456789012:role/*',
			'arn:aws:iam::123456789012:role/app/*',
			'arn:aws:iam::123456789012:role/application_abc/component_xyz/RDSAccess',
			'arn:aws:iam::123456789012:role/aws-service-role/access-analyzer.amazonaws.com/AWSServiceRoleForAccessAnalyzer',
			'arn:aws:iam::123456789012:role/service-role/QuickSightAction',
			'arn:aws:iam::123456789012:policy/division_abc/subdivision_xyz/UsersManageOwnCredentials',
			'arn:aws:iam::123456789012:policy/UsersManageOwnCredentials',
			'arn:aws-cn:organizations::123456789012:organization/o-123123123123',
			'arn:aws-cn:organizations::123456789012:ou/o-123123123123/ou-1234567890123',
			'arn:aws:sts::123456789012:assumed-role/demo/TestAR',
			'arn:aws:sqs:us-west-2:123456789012:mySQSqueue',
			'arn:aws:sqs:us-east-2:123456789012:mystack-myqueue-15PG5C2FC1CW8'
        ]:
            self.assertTrue(is_aws_arn(i))
        print_test_ok()

    def test_invalid_aws_arn(self):
        for i in [
            'arn:aws:iam::1234567890123:user/someName2',
			'arn:aws:iam::123456789012:user/Development//product_1234/*',
			'arn:aws:iam::123456789012:group//Development/product_1234/*',
			'arn:aws:iam::123456789012:group//Development/*/product_1234/*',
			'arn:aws:iam::1234567890123:role/role',
        ]:
            self.assertFalse(is_aws_arn(i))
        print_test_ok()

    def test_valid_aws_arn_iam_user(self):
        for i in [
			'arn:aws:iam::123456789012:user/someName1',
			'arn:aws:iam::123456789012:user/someName2',
			'arn:aws:iam::123456789012:user/user@mail.com',
			'arn:aws:iam::123456789012:user/*',
			'arn:aws:iam::123456789012:user/Development/product_1234/*'
        ]:
            self.assertTrue(is_aws_arn(i, "iam-user"))
        print_test_ok()

    def test_invalid_aws_arn_iam_user(self):
        for i in [
            'arn:aws:iam::1234567890123:user/someName2',
            'arn:aws:iam::123456789012:group/someName2',
			'arn:aws:iam::123456789012:user/Development//product_1234/*',
        ]:
            self.assertFalse(is_aws_arn(i, "iam-user"))
        print_test_ok()

    def test_valid_aws_arn_iam_group(self):
        for i in [
			'arn:aws:iam::123456789012:group/groupName1',
			'arn:aws:iam::123456789012:group/groupName2',
			'arn:aws:iam::123456789012:group/*',
			'arn:aws:iam::123456789012:group/Development/product_1234/*',
			'arn:aws:iam::123456789012:group/division_abc/subdivision_xyz/product_A/Developers',
			'arn:aws:iam::123456789012:group/Developers'
        ]:
            self.assertTrue(is_aws_arn(i, "iam-group"))
        print_test_ok()

    def test_invalid_aws_arn_iam_group(self):
        for i in [
            'arn:aws:iam::1234567890123:group/groupName1',
            'arn:aws:iam::123456789012:user/groupName1',
			'arn:aws:iam::123456789012:group//Development/product_1234/*',
			'arn:aws:iam::123456789012:group//Development/*/product_1234/*'
        ]:
            self.assertFalse(is_aws_arn(i, "iam-group"))
        print_test_ok()

    def test_valid_aws_arn_iam_role(self):
        for i in [
			'arn:aws:iam::123456789012:role/TestRole',
			'arn:aws:iam::123456789012:role/*',
			'arn:aws:iam::123456789012:role/app/*',
			'arn:aws:iam::123456789012:role/application_abc/component_xyz/RDSAccess',
			'arn:aws:iam::123456789012:role/aws-service-role/access-analyzer.amazonaws.com/AWSServiceRoleForAccessAnalyzer',
			'arn:aws:iam::123456789012:role/service-role/QuickSightAction'
        ]:
            self.assertTrue(is_aws_arn(i, "iam-role"))
        print_test_ok()

    def test_invalid_aws_arn_iam_role(self):
        for i in [
            'arn:aws:iam::1234567890123:role/role',
            'arn:aws:iam::123456789012:role',
			'arn:aws:iam::123456789012:group',
			'arn:aws:iam::123456789012:role//',
			'arn:aws:iam::123456789012:role/*/*',
			'arn:aws:iam::123456789012:role/*/application',
			'arn:aws:iam::123456789012:role/*/application/*',
        ]:
            self.assertFalse(is_aws_arn(i, "iam-role"))
        print_test_ok()

    def test_valid_aws_arn_iam_policy(self):
        for i in [
			'arn:aws:iam::123456789012:policy/UsersManageOwnCredentials',
			'arn:aws-cn:iam::123456789012:policy/UsersManageOwnCredentials',
			'arn:aws:iam::123456789012:policy/division_abc/subdivision_xyz/UsersManageOwnCredentials',
        ]:
            self.assertTrue(is_aws_arn(i, "iam-policy"))
        print_test_ok()

    def test_invalid_aws_arn_iam_policy(self):
        for i in [
            'arn:aws:iam::123456789012:role/TestRole',
			'arn:aws:iam::123456789012:policy'
			'arn:aws:iam::123456789012:policy/'
			'arn:aws:iam::123456789012:policy/*'
        ]:
            self.assertFalse(is_aws_arn(i, "iam-policy"))
        print_test_ok()

    def test_valid_aws_arn_organizations(self):
        for i in [
			'arn:aws:organizations::123456789012:account/o-123123123asd/123456789012',
			'arn:aws:organizations::123456789012:handshake/o-123123123asd/invite/h-123456789012',
			'arn:aws:organizations::123456789012:handshake/o-123123123asd/enable_all_features/h-123456789012',
			'arn:aws-cn:organizations::123456789012:organization/o-123123123123',
			'arn:aws:organizations::123456789012:ou/o-123123123asd/ou-123456789asd',
			'arn:aws-cn:organizations::123456789012:ou/o-123123123123/ou-1234567890123',
			'arn:aws-cn:organizations::123456789012:policy/o-123123123asd/aiservices_opt_out_policy/p-1234567890123',
        ]:
            self.assertTrue(is_aws_arn(i, "orgs"))
        print_test_ok()

    def test_invalid_aws_arn_organizations(self):
        for i in [
			'',
			'123',
			'asd',
			'arn:aws:iam::123456789012:policy',
			'not:an:arn::123456789012:ou/o-123457890asd/ou-123456789asd'
			'arn:aws:organizations',
			'arn:aws:organizations::123456789012:handshake/o-123123123asd/inviteee/h-123456789012',
			'arn:aws:iam::123456789012:role/*',
			'arn:aws:organizations::123456789012:organization/123123123asd',
			'arn::organizations::123456789012:account/o-123123123asd/123456789012',
			'arn:aws:organizations::123456789012:policy/o-123123123asd/asd/p-1234567890123',
        ]:
            self.assertFalse(is_aws_arn(i, "orgs"))
        print_test_ok()

    def test_valid_aws_arn_organizations_account(self):
        for i in [
			'arn:aws:organizations::123456789012:account/o-123123123asd/123456789012',
        ]:
            self.assertTrue(is_aws_arn(i, "orgs-account"))
        print_test_ok()

    def test_invalid_aws_arn_organizations_account(self):
        for i in [
			'arn:aws:organizations::123456789012:handshake/o-123123123asd/invite/h-123456789012',
			'arn:aws:iam::123456789012:role/*',
			'arn:aws:organizations::123456789012:account/o-123123123asd/12345678901',
			'arn:aws:organizations::123456789012:account/o-123123123asd/1234567890123',
			'arn:aws-cn:organizations::123456789012:account/o-123456789/123456789012',
        ]:
            self.assertFalse(is_aws_arn(i, "orgs-account"))
        print_test_ok()

    def test_valid_aws_arn_organizations_handshake(self):
        for i in [
			'arn:aws:organizations::123456789012:handshake/o-123123123asd/invite/h-123456789012',
			'arn:aws:organizations::123456789012:handshake/o-123123123asd/enable_all_features/h-123456789012',
			'arn:aws:organizations::123456789012:handshake/o-123123123asd/enable_all_features/h-123456789012ASD',
			'arn:aws:organizations::123456789012:handshake/o-123123123asd/approve_all_features/h-123456789012',
			'arn:aws:organizations::123456789012:handshake/o-123123123asd/add_organizations_service_linked_role/h-123456789012',
        ]:
            self.assertTrue(is_aws_arn(i, "orgs-handshake"))
        print_test_ok()

    def test_invalid_aws_arn_organizations_handshake(self):
        for i in [
            'arn:aws:organizations::123456789012:organization/o-123/invite/h-123456789012',
            'arn:aws:organizations::123456789012:organization/o-123123123asd/asd/h-123456789012'
            'arn:aws:organizations::1234567890:organization/o-123123123asd/invite/h-123456789012'
        ]:
            self.assertFalse(is_aws_arn(i, "orgs-handshake"))
        print_test_ok()

    def test_valid_aws_arn_organizations_organization(self):
        for i in [
			'arn:aws:organizations::123456789012:organization/o-123123123asd',
			'arn:aws-cn:organizations::123456789012:organization/o-123123123123',
        ]:
            self.assertTrue(is_aws_arn(i, "orgs-org"))
        print_test_ok()

    def test_invalid_aws_arn_organizations_organization(self):
        for i in [
            'arn:aws:organizations::123456789012:organization/o-123456789',
            'arn:aws:organizations::12345678901:organization/o-123456789012',
            'arn:aws:organizations::123456789012:organization/o-123456789asd/',
            'arn:aws:organizations::123456789012:organization/o-123123123asd/asd',
            'arn:aws:organizations::123456789012:organization/123123123asd'
        ]:
            self.assertFalse(is_aws_arn(i, "orgs-org"))
        print_test_ok()

    def test_valid_aws_arn_organizations_organization_units(self):
        for i in [
			'arn:aws:organizations::123456789012:ou/o-123123123asd/ou-123456789asd',
			'arn:aws-cn:organizations::123456789012:ou/o-123123123123/ou-1234567890123',
        ]:
            self.assertTrue(is_aws_arn(i, "orgs-unit"))
        print_test_ok()

    def test_invalid_aws_arn_organizations_organization_units(self):
        for i in [
            'arn:aws:organizations::123456789012:ou/o-123123123asd',
			'arn:aws-cn:organizations::123456789012:ou/o-123123123123/o-1234567890123',
			'arn:aws:organizations::123456789012:organization/o-123123123asd/ou-123456789asd'
        ]:
            self.assertFalse(is_aws_arn(i, "orgs-unit"))
        print_test_ok()

    def test_valid_aws_arn_organizations_policy(self):
        for i in [
			'arn:aws-cn:organizations::123456789012:policy/o-123123123asd/aiservices_opt_out_policy/p-1234567890123',
			'arn:aws-cn:organizations::123456789012:policy/o-123123123asd/tag_policy/p-1234567890123',
			'arn:aws-cn:organizations::123456789012:policy/o-123123123asd/SERVICE_CONTROL_POLICY/p-1234567890123',
			'arn:aws-cn:organizations::123456789012:policy/o-123123123asd/backup_policy/p-1234567890123',
        ]:
            self.assertTrue(is_aws_arn(i, "orgs-policy"))
        print_test_ok()

    def test_invalid_aws_arn_organizations_policy(self):
        for i in [
			'arn:aws:organizations::123456789012:policy/o-123123123asd/asd/p-1234567890123',
			'arn:aws-cn:organizations::123456789012:policy/o-123123123/aiservices_opt_out_policy/p-1234567890123',
			'arn:aws-cn:organizations::123456789012:policy/o-123123123asd/aiservices_opt_out_policy/1234567890123',
			'arn:aws-cn:organizations::123456789012:ou/o-123123123asd/aiservices_opt_out_policy/p-1234567890123',
        ]:
            self.assertFalse(is_aws_arn(i, "orgs-policy"))
        print_test_ok()

    def test_valid_aws_arn_s3(self):
        for i in [
			'arn:aws:s3:::my_corporate_bucket/*',
			'arn:aws:s3:::my_corporate_bucket/Development/*',
			'arn:aws:s3:::john/User image.png',
			'arn:aws:s3:::john',
			'arn:aws:s3:::test-bucket',
        ]:
            self.assertTrue(is_aws_arn(i, "s3"))
        print_test_ok()

    def test_invalid_aws_arn_s3(self):
        for i in [
			'arn:aws:s2:::john',
			'arn:aw:s3:::john',
			'arn:aws:s3::123456789012:john',
			'arn:aws:s3:::john/dir/img.jpg',
        ]:
            self.assertFalse(is_aws_arn(i, "s3"))
        print_test_ok()

    def test_valid_aws_arn_ecs(self):
        for i in [
			'arn:aws:ecs:us-west-2:012345678910:container-instance/f9cc75bb-0c94-46b9-bf6d-49d320bc1551',
			'arn:aws:ecs:us-west-2:012345678910:container-instance/cluster-name/f9cc75bb-0c94-46b9-bf6d-49d320bc1551',
			'arn:aws:ecs:us-west-2:012345678910:service/f9cc75bb-0c94-46b9-bf6d-49d320bc1551',
			'arn:aws:ecs:us-west-2:012345678910:service/cluster-name/f9cc75bb-0c94-46b9-bf6d-49d320bc1551',
			'arn:aws:ecs:us-west-2:012345678910:task/f9cc75bb-0c94-46b9-bf6d-49d320bc1551'
        ]:
            self.assertTrue(is_aws_arn(i, "ecs"))
        print_test_ok()

    def test_invalid_aws_arn_ecs(self):
        for i in [
			'arn:aws:ecs:us-west-2:012345678910:container-instance/asd/f9cc75bb-0c94-46b9-bf6d-49d320bc1551',
			'arn:aws:ecs:us-west-2:012345678910:container-instance',
			'arn:aws:ecs:us-west-2:012345678910:service/asd/f9cc75bb-0c94-46b9-bf6d-49d320bc1551',
			'arn:aws:ecs:us-west-2:012345678910:service',
        ]:
            self.assertFalse(is_aws_arn(i, "ecs"))
        print_test_ok()

    def test_valid_aws_arn_ecs_instance(self):
        for i in [
			'arn:aws:ecs:us-west-2:012345678910:container-instance/f9cc75bb-0c94-46b9-bf6d-49d320bc1551',
			'arn:aws:ecs:us-west-2:012345678910:container-instance/cluster-name/f9cc75bb-0c94-46b9-bf6d-49d320bc1551',
        ]:
            self.assertTrue(is_aws_arn(i, "ecs-instance"))
        print_test_ok()

    def test_invalid_aws_arn_ecs_instance(self):
        for i in [
			'arn:aws:ecs:us-west-2:012345678910:container-instance/asd/f9cc75bb-0c94-46b9-bf6d-49d320bc1551',
			'arn:aws:ecs:us-west-2:012345678910:container-instance'
        ]:
            self.assertFalse(is_aws_arn(i, "ecs-instance"))
        print_test_ok()

    def test_valid_aws_arn_ecs_service(self):
        for i in [
			'arn:aws:ecs:us-west-2:012345678910:service/f9cc75bb-0c94-46b9-bf6d-49d320bc1551',
			'arn:aws:ecs:us-west-2:012345678910:service/cluster-name/f9cc75bb-0c94-46b9-bf6d-49d320bc1551',
        ]:
            self.assertTrue(is_aws_arn(i, "ecs-service"))
        print_test_ok()

    def test_invalid_aws_arn_ecs_service(self):
        for i in [
			'arn:aws:ecs:us-west-2:012345678910:service/asd/f9cc75bb-0c94-46b9-bf6d-49d320bc1551',
			'arn:aws:ecs:us-west-2:012345678910:container-instance/f9cc75bb-0c94-46b9-bf6d-49d320bc1551',
			'arn:aws:ecs:us-west-2:012345678910:service'
        ]:
            self.assertFalse(is_aws_arn(i, "ecs-service"))
        print_test_ok()

    def test_valid_aws_arn_ecs_task(self):
        for i in [
			'arn:aws:ecs:us-west-2:012345678910:task/f9cc75bb-0c94-46b9-bf6d-49d320bc1551'
        ]:
            self.assertTrue(is_aws_arn(i, "ecs-task"))
        print_test_ok()

    def test_invalid_aws_arn_ecs_task(self):
        for i in [
			'arn:aws:ecs:us-west-2:012345678910:service/f9cc75bb-0c94-46b9-bf6d-49d320bc1551',
			'arn:aws:ecs:us-west-2:012345678910:task/asd/f9cc75bb-0c94-46b9-bf6d-49d320bc1551',
			'arn:aws:ecs:us-west-2:012345678910:container-instance/f9cc75bb-0c94-46b9-bf6d-49d320bc1551',
			'arn:aws:ecs:us-west-2:012345678910:service'
        ]:
            self.assertFalse(is_aws_arn(i, "ecs-task"))
        print_test_ok()

    def test_valid_aws_arn_ec2(self):
        for i in [
			'arn:aws:ec2:us-east-1:4575734578134:instance/i-054dsfg34gdsfg38',
			'arn:aws:ec2:us-west-2:12345:instance/i-060d319e74305c009'
        ]:
            self.assertTrue(is_aws_arn(i, "ec2"))
        print_test_ok()

    def test_invalid_aws_arn_ec2(self):
        for i in [
			'arn:aws:ec2:us-west-2:12345:instanc/i-060d319e74305c009',
			'arn:aws:ec2:us-west-2:12345:instance/n-060d319e74305c009'
        ]:
            self.assertFalse(is_aws_arn(i, "ec2"))
        print_test_ok()

    def test_valid_aws_arn_lambda(self):
        for i in [
			'arn:aws:lambda:aws-region:123456789012:function:helloworld:42',
			'arn:aws:lambda:aws-region:123456789012:function:source_lambda',
			'arn:aws:lambda:us-west-2:123456789012:event-source-mapping/f0acb93d',
			'arn:aws:lambda:us-east-2:123456789012:layer:my-layer',
			'arn:aws:lambda:us-east-2:123456789012:layer:my-layer:1'
        ]:
            self.assertTrue(is_aws_arn(i, "lambda"))
        print_test_ok()

    def test_invalid_aws_arn_lambda(self):
        for i in [
			'arn:aws:iam:us-west-2:123456789012:function:my-function:1',
			'arn:aws:lambda:us-west-2:123456789012:function:my-function:prod:asd',
			'arn:aws:lambda:us-east-2:123456789012:laye:my-layer:1',
			'arn:aws:ec2:us-east-1:4575734578134:instance/i-054dsfg34gdsfg38',
			'arn:aws:ec2:us-west-2:12345:instance/i-060d319e74305c009',
			'arn:aws:ecs:us-west-2:012345678910:service/f9cc75bb-0c94-46b9-bf6d-49d320bc1551',
			'arn:aws:ecs:us-west-2:012345678910:service/cluster-name/f9cc75bb-0c94-46b9-bf6d-49d320bc1551',
			'arn:aws:iam::123456789012:role/aws-service-role/access-analyzer.amazonaws.com/AWSServiceRoleForAccessAnalyzer',
			'arn:aws:iam::123456789012:role/service-role/QuickSightAction',
			'arn:aws:iam::123456789012:group/division_abc/subdivision_xyz/product_A/Developers',
			'arn:aws:iam::123456789012:group/Developers'
        ]:
            self.assertFalse(is_aws_arn(i, "lambda"))
        print_test_ok()

    def test_valid_aws_arn_lambda_function(self):
        for i in [
			'arn:aws:lambda:us-west-2:123456789012:function:my-function:1',
			'arn:aws:lambda:aws-region:123456789012:function:helloworld:42',
			'arn:aws:lambda:aws-region:123456789012:function:source_lambda',
			'arn:aws:lambda:us-west-2:123456789012:function:my-function:prod'
        ]:
            self.assertTrue(is_aws_arn(i, "lambda-function"))
        print_test_ok()

    def test_invalid_aws_arn_lambda_function(self):
        for i in [
			'arn:aws:iam:us-west-2:123456789012:function:my-function:1',
			'arn:aws:lambda:us-west-2:123456789012:function:my-function:prod:asd',
			'arn:aws:lambda:us-west-2:123456789012:function:my-$function:prod',
			'arn:aws:lambda:us-west-2:1234567890123:functio/n:my-function:prod',
        ]:
            self.assertFalse(is_aws_arn(i, "lambda-function"))
        print_test_ok()

    def test_valid_aws_arn_lambda_event_mapping(self):
        for i in [
			'arn:aws:lambda:us-west-2:123456789012:event-source-mapping/f0acb93d',
        ]:
            self.assertTrue(is_aws_arn(i, "lambda-event-mapping"))
        print_test_ok()

    def test_invalid_aws_arn_lambda_event_mapping(self):
        for i in [
			'arn:aws:lambda:us-west-2:123456789012:event-mapping/f0acb93d',
			'arn:aws:lambd:us-west-2:123456789012:event-source-mapping/f0acb93d',
			'arn:aws:lambda:us-west-2:1234567890123:event-source-mapping/f0acb93d',
			'arn:aws:iam:us-west-2:123456789012:function:my-function:1',
        ]:
            self.assertFalse(is_aws_arn(i, "lambda-event-mapping"))
        print_test_ok()

    def test_valid_aws_arn_lambda_layer(self):
        for i in [
			'arn:aws:lambda:us-east-2:123456789012:layer:my-layer',
			'arn:aws:lambda:us-east-2:123456789012:layer:my-layer:1'
        ]:
            self.assertTrue(is_aws_arn(i, "lambda-layer"))
        print_test_ok()

    def test_invalid_aws_arn_lambda_layer(self):
        for i in [
			'arn:aws:lambda:us-east-2:123456789012:layer:my-layer:asd',
			'arn:aws:lambda:us-east-2:123456789012:laye:my-layer:1',
			'arn:aws:lambda:us-west-2:123456789012:event-mapping/f0acb93d',
        ]:
            self.assertFalse(is_aws_arn(i, "lambda-layer"))
        print_test_ok()

    def test_valid_aws_arn_lambda_sts(self):
        for i in [
			'arn:aws:sts::123456789012:federated-user/Paulo',
			'arn:aws:sts::123456789012:assumed-role/Accounting-Role/Mary',
			'arn:aws:sts::123456789012:assumed-role/demo/TestAR'
        ]:
            self.assertTrue(is_aws_arn(i, "sts"))
        print_test_ok()

    def test_invalid_aws_arn_lambda_layer(self):
        for i in [
			'arn:aws:sts::123456789012:assumed-role/Accounting-Role/Mary/asd',
			'arn:aws:lambda::123456789012:assumed-role/Accounting-Role/Mary'
        ]:
            self.assertFalse(is_aws_arn(i, "sts"))
        print_test_ok()

    def test_valid_aws_arn_sqs(self):
        for i in [
			'arn:aws:sqs:us-west-2:123456789012:mySQSqueue',
			'arn:aws:sqs:us-east-2:123456789012:mystack-myqueue-15PG5C2FC1CW8',
        ]:
            self.assertTrue(is_aws_arn(i, "sqs"))
        print_test_ok()

    def test_invalid_aws_arn_sqs(self):
        for i in [
			'arn:aws:sqs:us-east-2:123456789012:mystack-myqueue-15PG5C2FC1CW8/asd',
			'arn:aws:sts:us-east-2:123456789012:mystack-myqueue-15PG5C2FC1CW8',
			'arn:aws:sts:us-east-2:123456789012:mystack-myqueue-15PG5C2FC1CW8/',
        ]:
            self.assertFalse(is_aws_arn(i, "sqs"))
        print_test_ok()



from typing import Union


def slice(val: str, start: int, end: Union[int, None]) -> str:
    if end is None:
        return val[start:]
    return val[start:end]


def slice_and_upper_case(val: str, start: int, end: Union[int, None]) -> str:
    sliced = slice(val, start, end)
    return sliced.upper()


def slice_and_lower_case(val: str, start: int, end: Union[int, None]) -> str:
    sliced = slice(val, start, end)
    return sliced.lower()
